/*
 * Copyright (c) 2024 Meltytech, LLC
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import Shotcut.Controls as Shotcut
import org.shotcut.qml as Shotcut


Shotcut.KeyframableFilter {
    id: gyroflowRoot
    property string defProjectPath: ''
    property double defSmoothness: 0.5
    property bool defOverview: false
    property bool blockUpdate: true

    function setControls() {
        blockUpdate = true;
        gyroflowFile.url = filter.get('Project');
        slider_smoothness.value = filter.get('Smoothness');
        checkbox_overview.checked = filter.get('Overview') == 1;
        blockUpdate = false;
    }

    Component.onCompleted: {
        //gyroflow project file is (by default) the video_file_name + .gyroflow extension, automatically assign this path:
        defProjectPath = producer.resource.substr(0, producer.resource.lastIndexOf(".")) + '.gyroflow';

        if (filter.isNew) {
            filter.set('Project', defProjectPath);
            filter.set('Smoothness', defSmoothness);
            filter.set('Overview', defOverview);
        }
        setControls();
        //console.log('dupa setControls in onCompleted: \n' + filter.get("Project") + '\n' + filter.get("Smoothness") + '\n' + filter.get("Overview"));
	}

    Shotcut.File {
        id: gyroflowFile
        onUrlChanged: {
            fileLabel.text = fileName;
            filter.set('Project', filePath);
            if (fileName.length > 0 && gyroflowFile.exists()) {
                fileLabel.color = activePalette.text;
                fileLabelTip.text = filePath;
            }
            else {
                fileLabel.color = 'red';
                fileLabelTip.text = qsTr('No matching Gyroflow file found.\nClick "Open" to load a new file.');
            }
        }
    }

    Shotcut.FileDialog {
        id: fileDialog
        title: qsTr("Select Gyroflow project file")
        nameFilters: ['Gyroflow project file (*.gyroflow)', 'All files (*.*)']
        onAccepted: {
            gyroflowFile.url = fileDialog.selectedFile;
            setControls();
        }
    }

    width: 300
    height: 100
    GridLayout {
        columns: 3
        anchors.fill: parent
        anchors.margins: 8
        Shotcut.Button {
            id: openButton
            text: qsTr('Open file')
            Layout.alignment: Qt.AlignRight
            onClicked: {
                fileDialog.title = qsTr("Open Gyroflow project file");
                //note: project file is usually in the same folder as the .mp4 so open dialog there
                settings.openPath = producer.resource;
                fileDialog.open();
            }
        }
        Label {
            id: fileLabel
            Layout.fillWidth: true
            Shotcut.HoverTip {
                id: fileLabelTip
            }
        }
        Shotcut.UndoButton {
            onClicked: {
                if (blockUpdate) return;
                gyroflowFile.url = defProjectPath
            }
        }

        Label {
            text: qsTr('Smoothness')
            Layout.alignment: Qt.AlignRight
        }
        Shotcut.SliderSpinner {
            id: slider_smoothness
            minimumValue: 0
            maximumValue: 1
            decimals: 2
            onValueChanged: {
                filter.set('Smoothness', slider_smoothness.value);
            }
        }
        Shotcut.UndoButton {
            onClicked: {
                if (blockUpdate) return;
                slider_smoothness.value = defSmoothness
            }
        }


        Label {
            text: qsTr('Overview')
            Layout.alignment: Qt.AlignRight

            Shotcut.HoverTip {
                text: qsTr('Overview will show the entire video area around stabilized part.')
            }
        }
        CheckBox {
            id: checkbox_overview

            leftPadding: 0
            onClicked: {
                filter.set('Overview', checked ? 1 : 0)
            }
        }
        Item {}

        Shotcut.TipBox {
            Layout.columnSpan: parent.columns
            Layout.fillWidth: true
            Layout.margins: 10
            text: qsTr('Tip: You must first open the video file in the Gyroflow app and save a project (Ctrl+S) after adjusting sync. Doing this before adding the filter will automatically detect and load the file.')
        }

        Item {
            Layout.fillHeight: true
        }
    }

}
